@@ -64,7 +64,7 @@
  * use or other dealings in this Software without prior written
  * authorization of the copyright holder.
  */
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "pango-engine.h"
@@ -412,7 +412,7 @@ get_index (PangoFcFont *fc_font, gunichar wc)
 
 
 static void
-tibetan_engine_shape (PangoEngineShape *engine,
+tibetan_engine_shape (PangoEngineShape *engine G_GNUC_UNUSED,
 		    PangoFont        *font,
 		    const char       *text,
 		    int               length,
@@ -518,7 +518,7 @@ tibetan_engine_shape (PangoEngineShape *engine,
   desc.n_other_features = 0;
   desc.other_features = NULL;
 
-  ruleset = pango_ot_ruleset_get_for (pango_ot_info_get (face), &desc);
+  ruleset = pango_ot_ruleset_get_for_description (pango_ot_info_get (face), &desc);
 
   pango_ot_ruleset_substitute (ruleset, buffer);
   pango_ot_ruleset_position (ruleset, buffer);
