@@ -1,15 +1,28 @@
-/*******************************************************************
- *
- *  Copyright 1996-2000 by
- *  David Turner, Robert Wilhelm, and Werner Lemberg.
- *
- *  Copyright 2006  Behdad Esfahbod
+/*
+ * Copyright (C) 1998-2004  David Turner and Werner Lemberg
+ * Copyright (C) 2006  Behdad Esfahbod
  *
  *  This is part of HarfBuzz, an OpenType Layout engine library.
  *
- *  See the file name COPYING for licensing information.
+ * Permission is hereby granted, without written agreement and without
+ * license or royalty fees, to use, copy, modify, and distribute this
+ * software and its documentation for any purpose, provided that the
+ * above copyright notice and the following two paragraphs appear in
+ * all copies of this software.
  *
- ******************************************************************/
+ * IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
+ * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
+ * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
+ * IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
+ * DAMAGE.
+ *
+ * THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
+ * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+ * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
+ * ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
+ * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
+ */
+
 #include "harfbuzz-impl.h"
 #include "harfbuzz-gdef-private.h"
 #include "harfbuzz-open-private.h"
@@ -26,94 +39,6 @@ static void  Free_NewGlyphClasses( HB_GDEFHeader*  gdef );
 
 
 
-/**********************
- * Extension Functions
- **********************/
-
-#if 0
-#define GDEF_ID  Build_Extension_ID( 'G', 'D', 'E', 'F' )
-
-
-static HB_Error  GDEF_Create( void*  ext,
-			      PFace  face )
-{
-  DEFINE_LOAD_LOCALS( face->stream );
-
-  HB_GDEFHeader*  gdef = (HB_GDEFHeader*)ext;
-  Long             table;
-
-
-  /* by convention */
-
-  if ( !gdef )
-    return HB_Err_Ok;
-
-  /* a null offset indicates that there is no GDEF table */
-
-  gdef->offset = 0;
-
-  /* we store the start offset and the size of the subtable */
-
-  table = HB_LookUp_Table( face, TTAG_GDEF );
-  if ( table < 0 )
-    return HB_Err_Ok;             /* The table is optional */
-
-  if ( FILE_Seek( face->dirTables[table].Offset ) ||
-       ACCESS_Frame( 4L ) )
-    return error;
-
-  gdef->offset  = FILE_Pos() - 4L;    /* undo ACCESS_Frame() */
-  gdef->Version = GET_ULong();
-
-  FORGET_Frame();
-
-  gdef->loaded = FALSE;
-
-  return HB_Err_Ok;
-}
-
-
-static HB_Error  GDEF_Destroy( void*  ext,
-			       PFace  face )
-{
-  HB_GDEFHeader*  gdef = (HB_GDEFHeader*)ext;
-
-
-  /* by convention */
-
-  if ( !gdef )
-    return HB_Err_Ok;
-
-  if ( gdef->loaded )
-  {
-    Free_LigCaretList( &gdef->LigCaretList );
-    Free_AttachList( &gdef->AttachList );
-    _HB_OPEN_Free_ClassDefinition( &gdef->GlyphClassDef );
-    _HB_OPEN_Free_ClassDefinition( &gdef->MarkAttachClassDef );
-
-    Free_NewGlyphClasses( gdef );
-  }
-
-  return HB_Err_Ok;
-}
-
-
-
-HB_Error  HB_Init_GDEF_Extension( HB_Engine  engine )
-{
-  PEngine_Instance  _engine = HANDLE_Engine( engine );
-
-
-  if ( !_engine )
-    return ERR(HB_Err_Invalid_Engine);
-
-  return  HB_Register_Extension( _engine,
-				 GDEF_ID,
-				 sizeof ( HB_GDEFHeader ),
-				 GDEF_Create,
-				 GDEF_Destroy );
-}
-#endif
 /* GDEF glyph classes */
 
 #define UNCLASSIFIED_GLYPH  0
@@ -154,11 +79,11 @@ HB_Error  HB_New_GDEF_Table( HB_GDEFHeader** retptr )
 }
 
 
-HB_Error  HB_Load_GDEF_Table( FT_Face          face,
+HB_Error  HB_Load_GDEF_Table( HB_Font          font,
 			      HB_GDEFHeader** retptr )
 {
   HB_Error         error;
-  HB_Stream        stream = face->stream;
+  HB_Stream        stream = font->stream;
   HB_UInt         cur_offset, new_offset, base_offset;
 
   HB_GDEFHeader*  gdef;
@@ -851,8 +776,6 @@ static HB_Error  Make_ClassRange( HB_ClassDefinition*  cd,
   crr[index].End   = end;
   crr[index].Class = class;
 
-  cd->Defined[class] = TRUE;
-
   return HB_Err_Ok;
 }
 
@@ -882,11 +805,6 @@ HB_Error  HB_GDEF_Build_ClassDefinition( HB_GDEFHeader*  gdef,
 
   gcd->ClassFormat = 2;
 
-  /* A GlyphClassDef table contains at most 5 different class values */
-
-  if ( ALLOC_ARRAY( gcd->Defined, 5, HB_Bool ) )
-    return error;
-
   gcd->cd.cd2.ClassRangeCount  = 0;
   gcd->cd.cd2.ClassRangeRecord = NULL;
 
@@ -1034,7 +952,6 @@ Fail3:
   FREE( gcd->cd.cd2.ClassRangeRecord );
 
 Fail4:
-  FREE( gcd->Defined );
   return error;
 }
 
@@ -1200,5 +1117,44 @@ _HB_GDEF_Check_Property( HB_GDEFHeader* gdef,
   return HB_Err_Ok;
 }
 
+HB_INTERNAL HB_Error
+_HB_GDEF_LoadMarkAttachClassDef_From_LookupFlags( HB_GDEFHeader* gdef,
+						  HB_Stream      stream,
+						  HB_Lookup*     lo,
+						  HB_UShort      num_lookups)
+{
+  HB_Error   error = HB_Err_Ok;
+  HB_UShort  i;
+
+  /* We now check the LookupFlags for values larger than 0xFF to find
+     out whether we need to load the `MarkAttachClassDef' field of the
+     GDEF table -- this hack is necessary for OpenType 1.2 tables since
+     the version field of the GDEF table hasn't been incremented.
+
+     For constructed GDEF tables, we only load it if
+     `MarkAttachClassDef_offset' is not zero (nevertheless, a build of
+     a constructed mark attach table is not supported currently).       */
+
+  if ( gdef &&
+       gdef->MarkAttachClassDef_offset && !gdef->MarkAttachClassDef.loaded )
+  {
+    for ( i = 0; i < num_lookups; i++ )
+    {
+
+      if ( lo[i].LookupFlag & HB_LOOKUP_FLAG_IGNORE_SPECIAL_MARKS )
+      {
+	if ( FILE_Seek( gdef->MarkAttachClassDef_offset ) ||
+	     ( error = _HB_OPEN_Load_ClassDefinition( &gdef->MarkAttachClassDef,
+					     256, stream ) ) != HB_Err_Ok )
+	  goto Done;
+
+	break;
+      }
+    }
+  }
+
+Done:
+  return error;
+}
 
 /* END */
